@@ -14,41 +14,52 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
+
+import logging
+from datetime import datetime
 from functools import wraps
-from typing import Callable, Dict, TypeVar, cast
+from typing import Any, Callable, Container, TypeVar, cast
 
 from pendulum.parsing import ParserError
 from sqlalchemy import text
+from sqlalchemy.orm.query import Query
 
 from airflow.api_connexion.exceptions import BadRequest
 from airflow.configuration import conf
 from airflow.utils import timezone
 
+log = logging.getLogger(__name__)
 
-def validate_istimezone(value):
-    """Validates that a datetime is not naive"""
+
+def validate_istimezone(value: datetime) -> None:
+    """Validates that a datetime is not naive."""
     if not value.tzinfo:
         raise BadRequest("Invalid datetime format", detail="Naive datetime is disallowed")
 
 
-def format_datetime(value: str):
+def format_datetime(value: str) -> datetime:
     """
+    Format datetime objects.
+
     Datetime format parser for args since connexion doesn't parse datetimes
     https://github.com/zalando/connexion/issues/476
 
     This should only be used within connection views because it raises 400
     """
     value = value.strip()
-    if value[-1] != 'Z':
-        value = value.replace(" ", '+')
+    if value[-1] != "Z":
+        value = value.replace(" ", "+")
     try:
         return timezone.parse(value)
     except (ParserError, TypeError) as err:
         raise BadRequest("Incorrect datetime argument", detail=str(err))
 
 
-def check_limit(value: int):
+def check_limit(value: int) -> int:
     """
+    Check the limit does not exceed configured value.
+
     This checks the limit passed to view and raises BadRequest if
     limit exceed user configured value
     """
@@ -56,6 +67,11 @@ def check_limit(value: int):
     fallback = conf.getint("api", "fallback_page_limit")
 
     if value > max_val:
+        log.warning(
+            "The limit param value %s passed in API exceeds the configured maximum page limit %s",
+            value,
+            max_val,
+        )
         return max_val
     if value == 0:
         return fallback
@@ -67,7 +83,7 @@ def check_limit(value: int):
 T = TypeVar("T", bound=Callable)
 
 
-def format_parameters(params_formatters: Dict[str, Callable[..., bool]]) -> Callable[[T], T]:
+def format_parameters(params_formatters: dict[str, Callable[[Any], Any]]) -> Callable[[T], T]:
     """
     Decorator factory that create decorator that convert parameters using given formatters.
 
@@ -76,7 +92,7 @@ def format_parameters(params_formatters: Dict[str, Callable[..., bool]]) -> Call
     :param params_formatters: Map of key name and formatter function
     """
 
-    def format_parameters_decorator(func: T):
+    def format_parameters_decorator(func: T) -> T:
         @wraps(func)
         def wrapped_function(*args, **kwargs):
             for key, formatter in params_formatters.items():
@@ -89,9 +105,14 @@ def format_parameters(params_formatters: Dict[str, Callable[..., bool]]) -> Call
     return format_parameters_decorator
 
 
-def apply_sorting(query, order_by, to_replace=None, allowed_attrs=None):
-    """Apply sorting to query"""
-    lstriped_orderby = order_by.lstrip('-')
+def apply_sorting(
+    query: Query,
+    order_by: str,
+    to_replace: dict[str, str] | None = None,
+    allowed_attrs: Container[str] | None = None,
+) -> Query:
+    """Apply sorting to query."""
+    lstriped_orderby = order_by.lstrip("-")
     if allowed_attrs and lstriped_orderby not in allowed_attrs:
         raise BadRequest(
             detail=f"Ordering with '{lstriped_orderby}' is disallowed or "
