@@ -14,7 +14,7 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""Database sub-commands"""
+"""Database sub-commands."""
 from __future__ import annotations
 
 import os
@@ -27,19 +27,19 @@ from airflow import settings
 from airflow.exceptions import AirflowException
 from airflow.utils import cli as cli_utils, db
 from airflow.utils.db import REVISION_HEADS_MAP
-from airflow.utils.db_cleanup import config_dict, run_cleanup
+from airflow.utils.db_cleanup import config_dict, drop_archived_tables, export_archived_records, run_cleanup
 from airflow.utils.process_utils import execute_interactive
 
 
 def initdb(args):
-    """Initializes the metadata database"""
+    """Initializes the metadata database."""
     print("DB: " + repr(settings.engine.url))
     db.initdb()
     print("Initialization done")
 
 
 def resetdb(args):
-    """Resets the metadata database"""
+    """Resets the metadata database."""
     print("DB: " + repr(settings.engine.url))
     if not (args.yes or input("This will drop existing tables if they exist. Proceed? (y/n)").upper() == "Y"):
         raise SystemExit("Cancelled")
@@ -48,7 +48,7 @@ def resetdb(args):
 
 @cli_utils.action_cli(check_db=False)
 def upgradedb(args):
-    """Upgrades the metadata database"""
+    """Upgrades the metadata database."""
     print("DB: " + repr(settings.engine.url))
     if args.to_revision and args.to_version:
         raise SystemExit("Cannot supply both `--to-revision` and `--to-version`.")
@@ -63,7 +63,7 @@ def upgradedb(args):
     if args.from_revision:
         from_revision = args.from_revision
     elif args.from_version:
-        if parse_version(args.from_version) < parse_version('2.0.0'):
+        if parse_version(args.from_version) < parse_version("2.0.0"):
             raise SystemExit("--from-version must be greater or equal to than 2.0.0")
         from_revision = REVISION_HEADS_MAP.get(args.from_version)
         if not from_revision:
@@ -81,14 +81,19 @@ def upgradedb(args):
     else:
         print("Generating sql for upgrade -- upgrade commands will *not* be submitted.")
 
-    db.upgradedb(to_revision=to_revision, from_revision=from_revision, show_sql_only=args.show_sql_only)
+    db.upgradedb(
+        to_revision=to_revision,
+        from_revision=from_revision,
+        show_sql_only=args.show_sql_only,
+        reserialize_dags=args.reserialize_dags,
+    )
     if not args.show_sql_only:
         print("Upgrades done")
 
 
 @cli_utils.action_cli(check_db=False)
 def downgrade(args):
-    """Downgrades the metadata database"""
+    """Downgrades the metadata database."""
     if args.to_revision and args.to_version:
         raise SystemExit("Cannot supply both `--to-revision` and `--to-version`.")
     if args.from_version and args.from_revision:
@@ -134,17 +139,17 @@ def downgrade(args):
 
 
 def check_migrations(args):
-    """Function to wait for all airflow migrations to complete. Used for launching airflow in k8s"""
+    """Function to wait for all airflow migrations to complete. Used for launching airflow in k8s."""
     db.check_migrations(timeout=args.migration_wait_timeout)
 
 
 @cli_utils.action_cli(check_db=False)
 def shell(args):
-    """Run a shell that allows to access metadata database"""
+    """Run a shell that allows to access metadata database."""
     url = settings.engine.url
     print("DB: " + repr(url))
 
-    if url.get_backend_name() == 'mysql':
+    if url.get_backend_name() == "mysql":
         with NamedTemporaryFile(suffix="my.cnf") as f:
             content = textwrap.dedent(
                 f"""
@@ -159,23 +164,23 @@ def shell(args):
             f.write(content.encode())
             f.flush()
             execute_interactive(["mysql", f"--defaults-extra-file={f.name}"])
-    elif url.get_backend_name() == 'sqlite':
+    elif url.get_backend_name() == "sqlite":
         execute_interactive(["sqlite3", url.database])
-    elif url.get_backend_name() == 'postgresql':
+    elif url.get_backend_name() == "postgresql":
         env = os.environ.copy()
-        env['PGHOST'] = url.host or ""
-        env['PGPORT'] = str(url.port or "5432")
-        env['PGUSER'] = url.username or ""
+        env["PGHOST"] = url.host or ""
+        env["PGPORT"] = str(url.port or "5432")
+        env["PGUSER"] = url.username or ""
         # PostgreSQL does not allow the use of PGPASSFILE if the current user is root.
         env["PGPASSWORD"] = url.password or ""
-        env['PGDATABASE'] = url.database
+        env["PGDATABASE"] = url.database
         execute_interactive(["psql"], env=env)
-    elif url.get_backend_name() == 'mssql':
+    elif url.get_backend_name() == "mssql":
         env = os.environ.copy()
-        env['MSSQL_CLI_SERVER'] = url.host
-        env['MSSQL_CLI_DATABASE'] = url.database
-        env['MSSQL_CLI_USER'] = url.username
-        env['MSSQL_CLI_PASSWORD'] = url.password
+        env["MSSQL_CLI_SERVER"] = url.host
+        env["MSSQL_CLI_DATABASE"] = url.database
+        env["MSSQL_CLI_USER"] = url.username
+        env["MSSQL_CLI_PASSWORD"] = url.password
         execute_interactive(["mssql-cli"], env=env)
     else:
         raise AirflowException(f"Unknown driver: {url.drivername}")
@@ -193,7 +198,7 @@ all_tables = sorted(config_dict)
 
 @cli_utils.action_cli(check_db=False)
 def cleanup_tables(args):
-    """Purges old records in metadata database"""
+    """Purges old records in metadata database."""
     run_cleanup(
         table_names=args.tables,
         dry_run=args.dry_run,
@@ -202,3 +207,24 @@ def cleanup_tables(args):
         confirm=not args.yes,
         skip_archive=args.skip_archive,
     )
+
+
+@cli_utils.action_cli(check_db=False)
+def export_archived(args):
+    """Exports archived records from metadata database."""
+    export_archived_records(
+        export_format=args.export_format,
+        output_path=args.output_path,
+        table_names=args.tables,
+        drop_archives=args.drop_archives,
+        needs_confirm=not args.yes,
+    )
+
+
+@cli_utils.action_cli(check_db=False)
+def drop_archived(args):
+    """Drops archived tables from metadata database."""
+    drop_archived_tables(
+        table_names=args.tables,
+        needs_confirm=not args.yes,
+    )
