@@ -16,7 +16,7 @@
 # specific language governing permissions and limitations
 # under the License.
 """
-DaskExecutor
+DaskExecutor.
 
 .. seealso::
     For more information on how the DaskExecutor works, take a look at the guide:
@@ -25,35 +25,41 @@ DaskExecutor
 from __future__ import annotations
 
 import subprocess
-from typing import Any
+from typing import TYPE_CHECKING, Any
 
 from distributed import Client, Future, as_completed
 from distributed.security import Security
 
 from airflow.configuration import conf
 from airflow.exceptions import AirflowException
-from airflow.executors.base_executor import NOT_STARTED_MESSAGE, BaseExecutor, CommandType
-from airflow.models.taskinstance import TaskInstanceKey
+from airflow.executors.base_executor import BaseExecutor
+
+if TYPE_CHECKING:
+    from airflow.executors.base_executor import CommandType
+    from airflow.models.taskinstancekey import TaskInstanceKey
+
 
 # queue="default" is a special case since this is the base config default queue name,
 # with respect to DaskExecutor, treat it as if no queue is provided
-_UNDEFINED_QUEUES = {None, 'default'}
+_UNDEFINED_QUEUES = {None, "default"}
 
 
 class DaskExecutor(BaseExecutor):
     """DaskExecutor submits tasks to a Dask Distributed cluster."""
 
+    supports_pickling: bool = False
+
     def __init__(self, cluster_address=None):
         super().__init__(parallelism=0)
         if cluster_address is None:
-            cluster_address = conf.get('dask', 'cluster_address')
+            cluster_address = conf.get("dask", "cluster_address")
         if not cluster_address:
-            raise ValueError('Please provide a Dask cluster address in airflow.cfg')
+            raise ValueError("Please provide a Dask cluster address in airflow.cfg")
         self.cluster_address = cluster_address
         # ssl / tls parameters
-        self.tls_ca = conf.get('dask', 'tls_ca')
-        self.tls_key = conf.get('dask', 'tls_key')
-        self.tls_cert = conf.get('dask', 'tls_cert')
+        self.tls_ca = conf.get("dask", "tls_ca")
+        self.tls_key = conf.get("dask", "tls_key")
+        self.tls_cert = conf.get("dask", "tls_cert")
         self.client: Client | None = None
         self.futures: dict[Future, TaskInstanceKey] | None = None
 
@@ -78,20 +84,19 @@ class DaskExecutor(BaseExecutor):
         queue: str | None = None,
         executor_config: Any | None = None,
     ) -> None:
+        if TYPE_CHECKING:
+            assert self.client
 
         self.validate_airflow_tasks_run_command(command)
 
         def airflow_run():
             return subprocess.check_call(command, close_fds=True)
 
-        if not self.client:
-            raise AirflowException(NOT_STARTED_MESSAGE)
-
         resources = None
         if queue not in _UNDEFINED_QUEUES:
             scheduler_info = self.client.scheduler_info()
             avail_queues = {
-                resource for d in scheduler_info['workers'].values() for resource in d['resources']
+                resource for d in scheduler_info["workers"].values() for resource in d["resources"]
             }
 
             if queue not in avail_queues:
@@ -102,8 +107,9 @@ class DaskExecutor(BaseExecutor):
         self.futures[future] = key  # type: ignore
 
     def _process_future(self, future: Future) -> None:
-        if not self.futures:
-            raise AirflowException(NOT_STARTED_MESSAGE)
+        if TYPE_CHECKING:
+            assert self.futures
+
         if future.done():
             key = self.futures[future]
             if future.exception():
@@ -117,23 +123,25 @@ class DaskExecutor(BaseExecutor):
             self.futures.pop(future)
 
     def sync(self) -> None:
-        if self.futures is None:
-            raise AirflowException(NOT_STARTED_MESSAGE)
+        if TYPE_CHECKING:
+            assert self.futures
+
         # make a copy so futures can be popped during iteration
         for future in self.futures.copy():
             self._process_future(future)
 
     def end(self) -> None:
-        if not self.client:
-            raise AirflowException(NOT_STARTED_MESSAGE)
-        if self.futures is None:
-            raise AirflowException(NOT_STARTED_MESSAGE)
+        if TYPE_CHECKING:
+            assert self.client
+            assert self.futures
+
         self.client.cancel(list(self.futures.keys()))
         for future in as_completed(self.futures.copy()):
             self._process_future(future)
 
     def terminate(self):
-        if self.futures is None:
-            raise AirflowException(NOT_STARTED_MESSAGE)
+        if TYPE_CHECKING:
+            assert self.futures
+
         self.client.cancel(self.futures.keys())
         self.end()
