@@ -15,8 +15,9 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
 
-from typing import Dict, Iterable, Optional, Tuple
+from typing import Any, Iterable
 
 from sqlalchemy import Column, Integer, String, Text, func
 from sqlalchemy.orm.session import Session
@@ -50,20 +51,20 @@ class Pool(Base):
     slots = Column(Integer, default=0)
     description = Column(Text)
 
-    DEFAULT_POOL_NAME = 'default_pool'
+    DEFAULT_POOL_NAME = "default_pool"
 
     def __repr__(self):
         return str(self.pool)
 
     @staticmethod
     @provide_session
-    def get_pools(session: Session = NEW_SESSION):
+    def get_pools(session: Session = NEW_SESSION) -> list[Pool]:
         """Get all pools."""
         return session.query(Pool).all()
 
     @staticmethod
     @provide_session
-    def get_pool(pool_name: str, session: Session = NEW_SESSION):
+    def get_pool(pool_name: str, session: Session = NEW_SESSION) -> Pool | None:
         """
         Get the Pool with specific pool name from the Pools.
 
@@ -75,7 +76,7 @@ class Pool(Base):
 
     @staticmethod
     @provide_session
-    def get_default_pool(session: Session = NEW_SESSION):
+    def get_default_pool(session: Session = NEW_SESSION) -> Pool | None:
         """
         Get the Pool of the default_pool from the Pools.
 
@@ -103,11 +104,17 @@ class Pool(Base):
 
     @staticmethod
     @provide_session
-    def create_or_update_pool(name: str, slots: int, description: str, session: Session = NEW_SESSION):
+    def create_or_update_pool(
+        name: str,
+        slots: int,
+        description: str,
+        session: Session = NEW_SESSION,
+    ) -> Pool:
         """Create a pool with given parameters or update it if it already exists."""
         if not name:
-            return
-        pool = session.query(Pool).filter_by(pool=name).first()
+            raise ValueError("Pool name must not be empty")
+
+        pool = session.query(Pool).filter_by(pool=name).one_or_none()
         if pool is None:
             pool = Pool(pool=name, slots=slots, description=description)
             session.add(pool)
@@ -116,12 +123,11 @@ class Pool(Base):
             pool.description = description
 
         session.commit()
-
         return pool
 
     @staticmethod
     @provide_session
-    def delete_pool(name: str, session: Session = NEW_SESSION):
+    def delete_pool(name: str, session: Session = NEW_SESSION) -> Pool:
         """Delete pool by a given name."""
         if name == Pool.DEFAULT_POOL_NAME:
             raise AirflowException(f"{Pool.DEFAULT_POOL_NAME} cannot be deleted")
@@ -141,7 +147,7 @@ class Pool(Base):
         *,
         lock_rows: bool = False,
         session: Session = NEW_SESSION,
-    ) -> Dict[str, PoolStats]:
+    ) -> dict[str, PoolStats]:
         """
         Get Pool stats (Number of Running, Queued, Open & Total tasks)
 
@@ -154,17 +160,17 @@ class Pool(Base):
         """
         from airflow.models.taskinstance import TaskInstance  # Avoid circular import
 
-        pools: Dict[str, PoolStats] = {}
+        pools: dict[str, PoolStats] = {}
 
         query = session.query(Pool.pool, Pool.slots)
 
         if lock_rows:
             query = with_row_locks(query, session=session, **nowait(session))
 
-        pool_rows: Iterable[Tuple[str, int]] = query.all()
+        pool_rows: Iterable[tuple[str, int]] = query.all()
         for (pool_name, total_slots) in pool_rows:
             if total_slots == -1:
-                total_slots = float('inf')  # type: ignore
+                total_slots = float("inf")  # type: ignore
             pools[pool_name] = PoolStats(total=total_slots, running=0, queued=0, open=0)
 
         state_count_by_pool = (
@@ -178,7 +184,7 @@ class Pool(Base):
             # Some databases return decimal.Decimal here.
             count = int(count)
 
-            stats_dict: Optional[PoolStats] = pools.get(pool_name)
+            stats_dict: PoolStats | None = pools.get(pool_name)
             if not stats_dict:
                 continue
             # TypedDict key must be a string literal, so we use if-statements to set value
@@ -195,21 +201,21 @@ class Pool(Base):
 
         return pools
 
-    def to_json(self):
+    def to_json(self) -> dict[str, Any]:
         """
         Get the Pool in a json structure
 
         :return: the pool object in json format
         """
         return {
-            'id': self.id,
-            'pool': self.pool,
-            'slots': self.slots,
-            'description': self.description,
+            "id": self.id,
+            "pool": self.pool,
+            "slots": self.slots,
+            "description": self.description,
         }
 
     @provide_session
-    def occupied_slots(self, session: Session = NEW_SESSION):
+    def occupied_slots(self, session: Session = NEW_SESSION) -> int:
         """
         Get the number of slots used by running/queued tasks at the moment.
 
@@ -221,13 +227,13 @@ class Pool(Base):
         return int(
             session.query(func.sum(TaskInstance.pool_slots))
             .filter(TaskInstance.pool == self.pool)
-            .filter(TaskInstance.state.in_(list(EXECUTION_STATES)))
+            .filter(TaskInstance.state.in_(EXECUTION_STATES))
             .scalar()
             or 0
         )
 
     @provide_session
-    def running_slots(self, session: Session = NEW_SESSION):
+    def running_slots(self, session: Session = NEW_SESSION) -> int:
         """
         Get the number of slots used by running tasks at the moment.
 
@@ -245,7 +251,7 @@ class Pool(Base):
         )
 
     @provide_session
-    def queued_slots(self, session: Session = NEW_SESSION):
+    def queued_slots(self, session: Session = NEW_SESSION) -> int:
         """
         Get the number of slots used by queued tasks at the moment.
 
@@ -262,6 +268,24 @@ class Pool(Base):
             or 0
         )
 
+    @provide_session
+    def scheduled_slots(self, session: Session = NEW_SESSION) -> int:
+        """
+        Get the number of slots scheduled at the moment.
+
+        :param session: SQLAlchemy ORM Session
+        :return: the number of scheduled slots
+        """
+        from airflow.models.taskinstance import TaskInstance  # Avoid circular import
+
+        return int(
+            session.query(func.sum(TaskInstance.pool_slots))
+            .filter(TaskInstance.pool == self.pool)
+            .filter(TaskInstance.state == State.SCHEDULED)
+            .scalar()
+            or 0
+        )
+
     @provide_session
     def open_slots(self, session: Session = NEW_SESSION) -> float:
         """
@@ -271,6 +295,5 @@ class Pool(Base):
         :return: the number of slots
         """
         if self.slots == -1:
-            return float('inf')
-        else:
-            return self.slots - self.occupied_slots(session)
+            return float("inf")
+        return self.slots - self.occupied_slots(session)
