@@ -14,7 +14,7 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""Serialized DAG and BaseOperator"""
+"""Serialized DAG and BaseOperator."""
 from __future__ import annotations
 
 import collections.abc
@@ -38,25 +38,31 @@ from airflow.compat.functools import cache
 from airflow.configuration import conf
 from airflow.datasets import Dataset
 from airflow.exceptions import AirflowException, RemovedInAirflow3Warning, SerializationError
+from airflow.jobs.job import Job
 from airflow.models.baseoperator import BaseOperator, BaseOperatorLink
 from airflow.models.connection import Connection
 from airflow.models.dag import DAG, create_timetable
+from airflow.models.dagrun import DagRun
 from airflow.models.expandinput import EXPAND_INPUT_EMPTY, ExpandInput, create_expand_input, get_map_type_key
 from airflow.models.mappedoperator import MappedOperator
 from airflow.models.operator import Operator
 from airflow.models.param import Param, ParamsDict
-from airflow.models.taskinstance import SimpleTaskInstance
+from airflow.models.taskinstance import SimpleTaskInstance, TaskInstance
 from airflow.models.taskmixin import DAGNode
 from airflow.models.xcom_arg import XComArg, deserialize_xcom_arg, serialize_xcom_arg
 from airflow.providers_manager import ProvidersManager
 from airflow.serialization.enums import DagAttributeTypes as DAT, Encoding
 from airflow.serialization.helpers import serialize_template_field
 from airflow.serialization.json_schema import Validator, load_dag_schema
-from airflow.settings import DAGS_FOLDER, json
+from airflow.serialization.pydantic.dag_run import DagRunPydantic
+from airflow.serialization.pydantic.dataset import DatasetPydantic
+from airflow.serialization.pydantic.job import JobPydantic
+from airflow.serialization.pydantic.taskinstance import TaskInstancePydantic
+from airflow.settings import _ENABLE_AIP_44, DAGS_FOLDER, json
 from airflow.timetables.base import Timetable
 from airflow.utils.code_utils import get_python_source
 from airflow.utils.docs import get_docs_url
-from airflow.utils.module_loading import as_importable_string, import_string
+from airflow.utils.module_loading import import_string, qualname
 from airflow.utils.operator_resources import Resources
 from airflow.utils.task_group import MappedTaskGroup, TaskGroup
 
@@ -108,6 +114,7 @@ def _get_default_mapped_partial() -> dict[str, Any]:
 
 
 def encode_relativedelta(var: relativedelta.relativedelta) -> dict[str, Any]:
+    """Encode a relativedelta object."""
     encoded = {k: v for k, v in var.__dict__.items() if not k.startswith("_") and v}
     if var.weekday and var.weekday.n:
         # Every n'th Friday for example
@@ -118,6 +125,7 @@ def encode_relativedelta(var: relativedelta.relativedelta) -> dict[str, Any]:
 
 
 def decode_relativedelta(var: dict[str, Any]) -> relativedelta.relativedelta:
+    """Dencode a relativedelta object."""
     if "weekday" in var:
         var["weekday"] = relativedelta.weekday(*var["weekday"])  # type: ignore
     return relativedelta.relativedelta(**var)
@@ -181,7 +189,7 @@ def _encode_timetable(var: Timetable) -> dict[str, Any]:
     can be completely controlled by a custom subclass.
     """
     timetable_class = type(var)
-    importable_string = as_importable_string(timetable_class)
+    importable_string = qualname(timetable_class)
     if _get_registered_timetable(importable_string) is None:
         raise _TimetableNotRegistered(importable_string)
     return {Encoding.TYPE: importable_string, Encoding.VAR: var.serialize()}
@@ -279,7 +287,7 @@ class BaseSerialization:
     _datetime_types = (datetime.datetime,)
 
     # Object types that are always excluded in serialization.
-    _excluded_types = (logging.Logger, Connection, type)
+    _excluded_types = (logging.Logger, Connection, type, property)
 
     _json_schema: Validator | None = None
 
@@ -311,9 +319,7 @@ class BaseSerialization:
 
     @classmethod
     def from_dict(cls, serialized_obj: dict[Encoding, Any]) -> BaseSerialization | dict | list | set | tuple:
-        """Deserializes a python dict stored with type decorators and
-        reconstructs all DAGs and operators it contains.
-        """
+        """Deserialize a dict of type decorators and reconstructs all DAGs and operators it contains."""
         return cls.deserialize(serialized_obj)
 
     @classmethod
@@ -356,7 +362,7 @@ class BaseSerialization:
     def serialize_to_json(
         cls, object_to_serialize: BaseOperator | MappedOperator | DAG, decorated_fields: set
     ) -> dict[str, Any]:
-        """Serializes an object to json"""
+        """Serializes an object to JSON."""
         serialized_object: dict[str, Any] = {}
         keys_to_serialize = object_to_serialize.get_serialized_fields()
         for key in keys_to_serialize:
@@ -384,7 +390,7 @@ class BaseSerialization:
 
     @classmethod
     def serialize(
-        cls, var: Any, *, strict: bool = False
+        cls, var: Any, *, strict: bool = False, use_pydantic_models: bool = False
     ) -> Any:  # Unfortunately there is no support for recursive types in mypy
         """Helper function of depth first search for serialization.
 
@@ -398,6 +404,11 @@ class BaseSerialization:
 
         :meta private:
         """
+        if use_pydantic_models and not _ENABLE_AIP_44:
+            raise RuntimeError(
+                "Setting use_pydantic_models = True requires AIP-44 (in progress) feature flag to be true. "
+                "This parameter will be removed eventually when new serialization is used by AIP-44"
+            )
         if cls._is_primitive(var):
             # enum.IntEnum is an int instance, it causes json dumps error so we use its value.
             if isinstance(var, enum.Enum):
@@ -405,10 +416,14 @@ class BaseSerialization:
             return var
         elif isinstance(var, dict):
             return cls._encode(
-                {str(k): cls.serialize(v, strict=strict) for k, v in var.items()}, type_=DAT.DICT
+                {
+                    str(k): cls.serialize(v, strict=strict, use_pydantic_models=use_pydantic_models)
+                    for k, v in var.items()
+                },
+                type_=DAT.DICT,
             )
         elif isinstance(var, list):
-            return [cls.serialize(v, strict=strict) for v in var]
+            return [cls.serialize(v, strict=strict, use_pydantic_models=use_pydantic_models) for v in var]
         elif var.__class__.__name__ == "V1Pod" and _has_kubernetes() and isinstance(var, k8s.V1Pod):
             json_pod = PodGenerator.serialize_pod(var)
             return cls._encode(json_pod, type_=DAT.POD)
@@ -433,12 +448,23 @@ class BaseSerialization:
         elif isinstance(var, set):
             # FIXME: casts set to list in customized serialization in future.
             try:
-                return cls._encode(sorted(cls.serialize(v, strict=strict) for v in var), type_=DAT.SET)
+                return cls._encode(
+                    sorted(
+                        cls.serialize(v, strict=strict, use_pydantic_models=use_pydantic_models) for v in var
+                    ),
+                    type_=DAT.SET,
+                )
             except TypeError:
-                return cls._encode([cls.serialize(v, strict=strict) for v in var], type_=DAT.SET)
+                return cls._encode(
+                    [cls.serialize(v, strict=strict, use_pydantic_models=use_pydantic_models) for v in var],
+                    type_=DAT.SET,
+                )
         elif isinstance(var, tuple):
             # FIXME: casts tuple to list in customized serialization in future.
-            return cls._encode([cls.serialize(v, strict=strict) for v in var], type_=DAT.TUPLE)
+            return cls._encode(
+                [cls.serialize(v, strict=strict, use_pydantic_models=use_pydantic_models) for v in var],
+                type_=DAT.TUPLE,
+            )
         elif isinstance(var, TaskGroup):
             return TaskGroupSerialization.serialize_task_group(var)
         elif isinstance(var, Param):
@@ -448,24 +474,47 @@ class BaseSerialization:
         elif isinstance(var, Dataset):
             return cls._encode(dict(uri=var.uri, extra=var.extra), type_=DAT.DATASET)
         elif isinstance(var, SimpleTaskInstance):
-            return cls._encode(cls.serialize(var.__dict__, strict=strict), type_=DAT.SIMPLE_TASK_INSTANCE)
+            return cls._encode(
+                cls.serialize(var.__dict__, strict=strict, use_pydantic_models=use_pydantic_models),
+                type_=DAT.SIMPLE_TASK_INSTANCE,
+            )
+        elif use_pydantic_models and _ENABLE_AIP_44:
+            if isinstance(var, Job):
+                return cls._encode(JobPydantic.from_orm(var).dict(), type_=DAT.BASE_JOB)
+            elif isinstance(var, TaskInstance):
+                return cls._encode(TaskInstancePydantic.from_orm(var).dict(), type_=DAT.TASK_INSTANCE)
+            elif isinstance(var, DagRun):
+                return cls._encode(DagRunPydantic.from_orm(var).dict(), type_=DAT.DAG_RUN)
+            elif isinstance(var, Dataset):
+                return cls._encode(DatasetPydantic.from_orm(var).dict(), type_=DAT.DATA_SET)
+            else:
+                return cls.default_serialization(strict, var)
         else:
-            log.debug("Cast type %s to str in serialization.", type(var))
-            if strict:
-                raise SerializationError("Encountered unexpected type")
-            return str(var)
+            return cls.default_serialization(strict, var)
 
     @classmethod
-    def deserialize(cls, encoded_var: Any) -> Any:
+    def default_serialization(cls, strict, var) -> str:
+        log.debug("Cast type %s to str in serialization.", type(var))
+        if strict:
+            raise SerializationError("Encountered unexpected type")
+        return str(var)
+
+    @classmethod
+    def deserialize(cls, encoded_var: Any, use_pydantic_models=False) -> Any:
         """Helper function of depth first search for deserialization.
 
         :meta private:
         """
         # JSON primitives (except for dict) are not encoded.
+        if use_pydantic_models and not _ENABLE_AIP_44:
+            raise RuntimeError(
+                "Setting use_pydantic_models = True requires AIP-44 (in progress) feature flag to be true. "
+                "This parameter will be removed eventually when new serialization is used by AIP-44"
+            )
         if cls._is_primitive(encoded_var):
             return encoded_var
         elif isinstance(encoded_var, list):
-            return [cls.deserialize(v) for v in encoded_var]
+            return [cls.deserialize(v, use_pydantic_models) for v in encoded_var]
 
         if not isinstance(encoded_var, dict):
             raise ValueError(f"The encoded_var should be dict and is {type(encoded_var)}")
@@ -473,7 +522,7 @@ class BaseSerialization:
         type_ = encoded_var[Encoding.TYPE]
 
         if type_ == DAT.DICT:
-            return {k: cls.deserialize(v) for k, v in var.items()}
+            return {k: cls.deserialize(v, use_pydantic_models) for k, v in var.items()}
         elif type_ == DAT.DAG:
             return SerializedDAG.deserialize_dag(var)
         elif type_ == DAT.OP:
@@ -492,9 +541,9 @@ class BaseSerialization:
         elif type_ == DAT.RELATIVEDELTA:
             return decode_relativedelta(var)
         elif type_ == DAT.SET:
-            return {cls.deserialize(v) for v in var}
+            return {cls.deserialize(v, use_pydantic_models) for v in var}
         elif type_ == DAT.TUPLE:
-            return tuple(cls.deserialize(v) for v in var)
+            return tuple(cls.deserialize(v, use_pydantic_models) for v in var)
         elif type_ == DAT.PARAM:
             return cls._deserialize_param(var)
         elif type_ == DAT.XCOM_REF:
@@ -503,6 +552,15 @@ class BaseSerialization:
             return Dataset(**var)
         elif type_ == DAT.SIMPLE_TASK_INSTANCE:
             return SimpleTaskInstance(**cls.deserialize(var))
+        elif use_pydantic_models and _ENABLE_AIP_44:
+            if type_ == DAT.BASE_JOB:
+                return JobPydantic.parse_obj(var)
+            elif type_ == DAT.TASK_INSTANCE:
+                return TaskInstancePydantic.parse_obj(var)
+            elif type_ == DAT.DAG_RUN:
+                return DagRunPydantic.parse_obj(var)
+            elif type_ == DAT.DATA_SET:
+                return DatasetPydantic.parse_obj(var)
         else:
             raise TypeError(f"Invalid type {type_!s} in deserialization.")
 
@@ -515,7 +573,6 @@ class BaseSerialization:
 
     @classmethod
     def _is_constructor_param(cls, attrname: str, instance: Any) -> bool:
-
         return attrname in cls._CONSTRUCTOR_PARAMS
 
     @classmethod
@@ -553,6 +610,8 @@ class BaseSerialization:
     @classmethod
     def _deserialize_param(cls, param_dict: dict):
         """
+        Workaround to serialize Param on older versions.
+
         In 2.2.0, Param attrs were assumed to be json-serializable and were not run through
         this class's ``serialize`` method.  So before running through ``deserialize``,
         we first verify that it's necessary to do.
@@ -582,7 +641,7 @@ class BaseSerialization:
 
     @classmethod
     def _serialize_params_dict(cls, params: ParamsDict | dict):
-        """Serialize Params dict for a DAG/Task"""
+        """Serialize Params dict for a DAG or task."""
         serialized_params = {}
         for k, v in params.items():
             # TODO: As of now, we would allow serialization of params which are of type Param only.
@@ -601,7 +660,7 @@ class BaseSerialization:
 
     @classmethod
     def _deserialize_params_dict(cls, encoded_params: dict) -> ParamsDict:
-        """Deserialize a DAG's Params dict"""
+        """Deserialize a DAG's Params dict."""
         op_params = {}
         for k, v in encoded_params.items():
             if isinstance(v, dict) and "__class" in v:
@@ -622,10 +681,10 @@ class DependencyDetector:
 
     @staticmethod
     def detect_task_dependencies(task: Operator) -> list[DagDependency]:
+        """Detects dependencies caused by tasks."""
         from airflow.operators.trigger_dagrun import TriggerDagRunOperator
         from airflow.sensors.external_task import ExternalTaskSensor
 
-        """Detects dependencies caused by tasks"""
         deps = []
         if isinstance(task, TriggerDagRunOperator):
             deps.append(
@@ -763,7 +822,9 @@ class SerializedBaseOperator(BaseOperator, BaseSerialization):
 
         if op.operator_extra_links:
             serialize_op["_operator_extra_links"] = cls._serialize_operator_extra_links(
-                op.operator_extra_links
+                op.operator_extra_links.__get__(op)
+                if isinstance(op.operator_extra_links, property)
+                else op.operator_extra_links
             )
 
         if include_deps:
@@ -1009,19 +1070,19 @@ class SerializedBaseOperator(BaseOperator, BaseSerialization):
             raise AirflowException("Can not load plugins")
 
         instances = set()
-        for qualname in set(deps):
+        for qn in set(deps):
             if (
-                not qualname.startswith("airflow.ti_deps.deps.")
-                and qualname not in plugins_manager.registered_ti_dep_classes
+                not qn.startswith("airflow.ti_deps.deps.")
+                and qn not in plugins_manager.registered_ti_dep_classes
             ):
                 raise SerializationError(
-                    f"Custom dep class {qualname} not deserialized, please register it through plugins."
+                    f"Custom dep class {qn} not deserialized, please register it through plugins."
                 )
 
             try:
-                instances.add(import_string(qualname)())
+                instances.add(import_string(qn)())
             except ImportError:
-                log.warning("Error importing dep %r", qualname, exc_info=True)
+                log.warning("Error importing dep %r", qn, exc_info=True)
         return instances
 
     @classmethod
@@ -1090,8 +1151,10 @@ class SerializedBaseOperator(BaseOperator, BaseSerialization):
     @classmethod
     def _serialize_operator_extra_links(cls, operator_extra_links: Iterable[BaseOperatorLink]):
         """
-        Serialize Operator Links. Store the import path of the OperatorLink and the arguments
-        passed to it. Example
+        Serialize Operator Links.
+
+        Store the import path of the OperatorLink and the arguments passed to it.
+        For example:
         ``[{'airflow.providers.google.cloud.operators.bigquery.BigQueryConsoleLink': {}}]``
 
         :param operator_extra_links: Operator Link
@@ -1110,6 +1173,15 @@ class SerializedBaseOperator(BaseOperator, BaseSerialization):
 
         return serialize_operator_extra_links
 
+    @classmethod
+    def serialize(cls, var: Any, *, strict: bool = False, use_pydantic_models: bool = False) -> Any:
+        # the wonders of multiple inheritance BaseOperator defines an instance method
+        return BaseSerialization.serialize(var=var, strict=strict, use_pydantic_models=use_pydantic_models)
+
+    @classmethod
+    def deserialize(cls, encoded_var: Any, use_pydantic_models: bool = False) -> Any:
+        return BaseSerialization.deserialize(encoded_var=encoded_var, use_pydantic_models=use_pydantic_models)
+
 
 class SerializedDAG(DAG, BaseSerialization):
     """
@@ -1161,13 +1233,14 @@ class SerializedDAG(DAG, BaseSerialization):
                 del serialized_dag["timetable"]
 
             serialized_dag["tasks"] = [cls.serialize(task) for _, task in dag.task_dict.items()]
+
             dag_deps = {
                 dep
                 for task in dag.task_dict.values()
                 for dep in SerializedBaseOperator.detect_dependencies(task)
             }
             dag_deps.update(DependencyDetector.detect_dag_dependencies(dag))
-            serialized_dag["dag_dependencies"] = [x.__dict__ for x in dag_deps]
+            serialized_dag["dag_dependencies"] = [x.__dict__ for x in sorted(dag_deps)]
             serialized_dag["_task_group"] = TaskGroupSerialization.serialize_task_group(dag.task_group)
 
             # Edge info in the JSON exactly matches our internal structure
@@ -1194,7 +1267,6 @@ class SerializedDAG(DAG, BaseSerialization):
             if k == "_downstream_task_ids":
                 v = set(v)
             elif k == "tasks":
-
                 SerializedBaseOperator._load_operator_extra_links = cls._load_operator_extra_links
 
                 v = {task["task_id"]: SerializedBaseOperator.deserialize_operator(task) for task in v}
@@ -1374,7 +1446,7 @@ class TaskGroupSerialization(BaseSerialization):
         return group
 
 
-@dataclass(frozen=True)
+@dataclass(frozen=True, order=True)
 class DagDependency:
     """Dataclass for representing dependencies between DAGs.
     These are calculated during serialization and attached to serialized DAGs.
@@ -1387,7 +1459,7 @@ class DagDependency:
 
     @property
     def node_id(self):
-        """Node ID for graph rendering"""
+        """Node ID for graph rendering."""
         val = f"{self.dependency_type}"
         if not self.dependency_type == "dataset":
             val += f":{self.source}:{self.target}"
