@@ -20,26 +20,32 @@ This module took inspiration from the community maintenance dag
 """
 from __future__ import annotations
 
+import csv
 import logging
+import os
 from contextlib import contextmanager
 from dataclasses import dataclass
 from typing import Any
 
 from pendulum import DateTime
-from sqlalchemy import and_, column, false, func, table, text
+from sqlalchemy import and_, column, false, func, inspect, table, text
 from sqlalchemy.exc import OperationalError, ProgrammingError
 from sqlalchemy.ext.compiler import compiles
 from sqlalchemy.orm import Query, Session, aliased
 from sqlalchemy.sql.expression import ClauseElement, Executable, tuple_
 
+from airflow import AirflowException
 from airflow.cli.simple_table import AirflowConsole
 from airflow.models import Base
 from airflow.utils import timezone
 from airflow.utils.db import reflect_tables
+from airflow.utils.helpers import ask_yesno
 from airflow.utils.session import NEW_SESSION, provide_session
 
 logger = logging.getLogger(__file__)
 
+ARCHIVE_TABLE_PREFIX = "_airflow_deleted__"
+
 
 @dataclass
 class _TableConfig:
@@ -123,15 +129,27 @@ def _check_for_rows(*, query: Query, print_rows=False):
     return num_entities
 
 
+def _dump_table_to_file(*, target_table, file_path, export_format, session):
+    if export_format == "csv":
+        with open(file_path, "w") as f:
+            csv_writer = csv.writer(f)
+            cursor = session.execute(text(f"SELECT * FROM {target_table}"))
+            csv_writer.writerow(cursor.keys())
+            csv_writer.writerows(cursor.fetchall())
+    else:
+        raise AirflowException(f"Export format {export_format} is not supported.")
+
+
 def _do_delete(*, query, orm_model, skip_archive, session):
-    import re
     from datetime import datetime
 
+    import re2
+
     print("Performing Delete...")
     # using bulk delete
     # create a new table and copy the rows there
-    timestamp_str = re.sub(r"[^\d]", "", datetime.utcnow().isoformat())[:14]
-    target_table_name = f"_airflow_deleted__{orm_model.name}__{timestamp_str}"
+    timestamp_str = re2.sub(r"[^\d]", "", datetime.utcnow().isoformat())[:14]
+    target_table_name = f"{ARCHIVE_TABLE_PREFIX}{orm_model.name}__{timestamp_str}"
     print(f"Moving data to table {target_table_name}")
     bind = session.get_bind()
     dialect_name = bind.dialect.name
@@ -255,7 +273,7 @@ def _cleanup_table(
     dry_run=True,
     verbose=False,
     skip_archive=False,
-    session=None,
+    session,
     **kwargs,
 ):
     print()
@@ -294,6 +312,26 @@ def _confirm_delete(*, date: DateTime, tables: list[str]):
         raise SystemExit("User did not confirm; exiting.")
 
 
+def _confirm_drop_archives(*, tables: list[str]):
+    # if length of tables is greater than 3, show the total count
+    if len(tables) > 3:
+        text_ = f"{len(tables)} archived tables prefixed with {ARCHIVE_TABLE_PREFIX}"
+    else:
+        text_ = f"the following archived tables {tables}"
+    question = (
+        f"You have requested that we drop {text_}.\n"
+        f"This is irreversible. Consider backing up the tables first \n"
+    )
+    print(question)
+    if len(tables) > 3:
+        show_tables = ask_yesno("Show tables? (y/n): ")
+        if show_tables:
+            print(tables, "\n")
+    answer = input("Enter 'drop archived tables' (without quotes) to proceed.\n").strip()
+    if not answer == "drop archived tables":
+        raise SystemExit("User did not confirm; exiting.")
+
+
 def _print_config(*, configs: dict[str, _TableConfig]):
     data = [x.readable_config for x in configs.values()]
     AirflowConsole().print_as_table(data=data)
@@ -315,6 +353,33 @@ def _suppress_with_logging(table, session):
             session.rollback()
 
 
+def _effective_table_names(*, table_names: list[str] | None):
+    desired_table_names = set(table_names or config_dict)
+    effective_config_dict = {k: v for k, v in config_dict.items() if k in desired_table_names}
+    effective_table_names = set(effective_config_dict)
+    if desired_table_names != effective_table_names:
+        outliers = desired_table_names - effective_table_names
+        logger.warning(
+            "The following table(s) are not valid choices and will be skipped: %s", sorted(outliers)
+        )
+    if not effective_table_names:
+        raise SystemExit("No tables selected for db cleanup. Please choose valid table names.")
+    return effective_table_names, effective_config_dict
+
+
+def _get_archived_table_names(table_names, session):
+    inspector = inspect(session.bind)
+    db_table_names = [x for x in inspector.get_table_names() if x.startswith(ARCHIVE_TABLE_PREFIX)]
+    effective_table_names, _ = _effective_table_names(table_names=table_names)
+    # Filter out tables that don't start with the archive prefix
+    archived_table_names = [
+        table_name
+        for table_name in db_table_names
+        if any("__" + x + "__" in table_name for x in effective_table_names)
+    ]
+    return archived_table_names
+
+
 @provide_session
 def run_cleanup(
     *,
@@ -346,8 +411,7 @@ def run_cleanup(
     :param session: Session representing connection to the metadata database.
     """
     clean_before_timestamp = timezone.coerce_datetime(clean_before_timestamp)
-    effective_table_names = table_names if table_names else list(config_dict.keys())
-    effective_config_dict = {k: v for k, v in config_dict.items() if k in effective_table_names}
+    effective_table_names, effective_config_dict = _effective_table_names(table_names=table_names)
     if dry_run:
         print("Performing dry run for db cleanup.")
         print(
@@ -356,7 +420,7 @@ def run_cleanup(
         )
         _print_config(configs=effective_config_dict)
     if not dry_run and confirm:
-        _confirm_delete(date=clean_before_timestamp, tables=list(effective_config_dict.keys()))
+        _confirm_delete(date=clean_before_timestamp, tables=sorted(effective_table_names))
     existing_tables = reflect_tables(tables=None, session=session).tables
     for table_name, table_config in effective_config_dict.items():
         if table_name not in existing_tables:
@@ -372,3 +436,50 @@ def run_cleanup(
                 session=session,
             )
             session.commit()
+
+
+@provide_session
+def export_archived_records(
+    export_format,
+    output_path,
+    table_names=None,
+    drop_archives=False,
+    needs_confirm=True,
+    session: Session = NEW_SESSION,
+):
+    """Export archived data to the given output path in the given format."""
+    archived_table_names = _get_archived_table_names(table_names, session)
+    # If user chose to drop archives, check there are archive tables that exists
+    # before asking for confirmation
+    if drop_archives and archived_table_names and needs_confirm:
+        _confirm_drop_archives(tables=sorted(archived_table_names))
+    export_count = 0
+    dropped_count = 0
+    for table_name in archived_table_names:
+        logger.info("Exporting table %s", table_name)
+        _dump_table_to_file(
+            target_table=table_name,
+            file_path=os.path.join(output_path, f"{table_name}.{export_format}"),
+            export_format=export_format,
+            session=session,
+        )
+        export_count += 1
+        if drop_archives:
+            logger.info("Dropping archived table %s", table_name)
+            session.execute(text(f"DROP TABLE {table_name}"))
+            dropped_count += 1
+    logger.info("Total exported tables: %s, Total dropped tables: %s", export_count, dropped_count)
+
+
+@provide_session
+def drop_archived_tables(table_names, needs_confirm, session):
+    """Drop archived tables."""
+    archived_table_names = _get_archived_table_names(table_names, session)
+    if needs_confirm and archived_table_names:
+        _confirm_drop_archives(tables=sorted(archived_table_names))
+    dropped_count = 0
+    for table_name in archived_table_names:
+        logger.info("Dropping archived table %s", table_name)
+        session.execute(text(f"DROP TABLE {table_name}"))
+        dropped_count += 1
+    logger.info("Total dropped tables: %s", dropped_count)
