@@ -17,15 +17,16 @@
 # under the License.
 from __future__ import annotations
 
+import ast
 import io
 import logging
 import os
-import re
 import zipfile
 from collections import OrderedDict
 from pathlib import Path
 from typing import TYPE_CHECKING, Generator, NamedTuple, Pattern, overload
 
+import re2
 from pathspec.patterns import GitWildMatchPattern
 from typing_extensions import Protocol
 
@@ -63,8 +64,8 @@ class _RegexpIgnoreRule(NamedTuple):
     def compile(pattern: str, base_dir: Path, definition_file: Path) -> _IgnoreRule | None:
         """Build an ignore rule from the supplied regexp pattern and log a useful warning if it is invalid"""
         try:
-            return _RegexpIgnoreRule(re.compile(pattern), base_dir)
-        except re.error as e:
+            return _RegexpIgnoreRule(re2.compile(pattern), base_dir)
+        except re2.error as e:
             log.warning("Ignoring invalid regex '%s' from %s: %s", pattern, definition_file, e)
             return None
 
@@ -153,7 +154,7 @@ def mkdirs(path, mode):
     Path(path).mkdir(mode=mode, parents=True, exist_ok=True)
 
 
-ZIP_REGEX = re.compile(fr'((.*\.zip){re.escape(os.sep)})?(.*)')
+ZIP_REGEX = re2.compile(rf"((.*\.zip){re2.escape(os.sep)})?(.*)")
 
 
 @overload
@@ -183,7 +184,7 @@ def correct_maybe_zipped(fileloc: None | str | Path) -> None | str | Path:
         return fileloc
 
 
-def open_maybe_zipped(fileloc, mode='r'):
+def open_maybe_zipped(fileloc, mode="r"):
     """
     Opens the given file. If the path contains a folder with a .zip suffix, then
     the folder is treated as a zip archive, opening the file inside the archive.
@@ -194,7 +195,6 @@ def open_maybe_zipped(fileloc, mode='r'):
     if archive and zipfile.is_zipfile(archive):
         return io.TextIOWrapper(zipfile.ZipFile(archive, mode=mode).open(filename))
     else:
-
         return open(fileloc, mode=mode)
 
 
@@ -221,7 +221,7 @@ def _find_path_from_directory(
         ignore_file_path = Path(root) / ignore_file_name
         if ignore_file_path.is_file():
             with open(ignore_file_path) as ifile:
-                lines_no_comments = [re.sub(r"\s*#.*", "", line) for line in ifile.read().split("\n")]
+                lines_no_comments = [re2.sub(r"\s*#.*", "", line) for line in ifile.read().split("\n")]
                 # append new patterns and filter out "None" objects, which are invalid patterns
                 patterns += [
                     p
@@ -260,7 +260,7 @@ def _find_path_from_directory(
 def find_path_from_directory(
     base_dir_path: str,
     ignore_file_name: str,
-    ignore_file_syntax: str = conf.get_mandatory_value('core', 'DAG_IGNORE_FILE_SYNTAX', fallback="regexp"),
+    ignore_file_syntax: str = conf.get_mandatory_value("core", "DAG_IGNORE_FILE_SYNTAX", fallback="regexp"),
 ) -> Generator[str, None, None]:
     """
     Recursively search the base path and return the list of file paths that should not be ignored.
@@ -280,9 +280,9 @@ def find_path_from_directory(
 
 def list_py_file_paths(
     directory: str | pathlib.Path,
-    safe_mode: bool = conf.getboolean('core', 'DAG_DISCOVERY_SAFE_MODE', fallback=True),
+    safe_mode: bool = conf.getboolean("core", "DAG_DISCOVERY_SAFE_MODE", fallback=True),
     include_examples: bool | None = None,
-):
+) -> list[str]:
     """
     Traverse a directory and look for Python files.
 
@@ -293,10 +293,9 @@ def list_py_file_paths(
         to safe.
     :param include_examples: include example DAGs
     :return: a list of paths to Python files in the specified directory
-    :rtype: list[unicode]
     """
     if include_examples is None:
-        include_examples = conf.getboolean('core', 'LOAD_EXAMPLES')
+        include_examples = conf.getboolean("core", "LOAD_EXAMPLES")
     file_paths: list[str] = []
     if directory is None:
         file_paths = []
@@ -321,7 +320,7 @@ def find_dag_file_paths(directory: str | pathlib.Path, safe_mode: bool) -> list[
             if not os.path.isfile(file_path):
                 continue
             _, file_ext = os.path.splitext(os.path.split(file_path)[-1])
-            if file_ext != '.py' and not zipfile.is_zipfile(file_path):
+            if file_ext != ".py" and not zipfile.is_zipfile(file_path):
                 continue
             if not might_contain_dag(file_path, safe_mode):
                 continue
@@ -333,27 +332,62 @@ def find_dag_file_paths(directory: str | pathlib.Path, safe_mode: bool) -> list[
     return file_paths
 
 
-COMMENT_PATTERN = re.compile(r"\s*#.*")
+COMMENT_PATTERN = re2.compile(r"\s*#.*")
 
 
-def might_contain_dag(file_path: str, safe_mode: bool, zip_file: zipfile.ZipFile | None = None):
+def might_contain_dag(file_path: str, safe_mode: bool, zip_file: zipfile.ZipFile | None = None) -> bool:
+    """
+    Check whether a Python file contains Airflow DAGs.
+    When safe_mode is off (with False value), this function always returns True.
+
+    If might_contain_dag_callable isn't specified, it uses airflow default heuristic
+    """
+    if not safe_mode:
+        return True
+
+    might_contain_dag_callable = conf.getimport(
+        "core",
+        "might_contain_dag_callable",
+        fallback="airflow.utils.file.might_contain_dag_via_default_heuristic",
+    )
+    return might_contain_dag_callable(file_path=file_path, zip_file=zip_file)
+
+
+def might_contain_dag_via_default_heuristic(file_path: str, zip_file: zipfile.ZipFile | None = None) -> bool:
     """
     Heuristic that guesses whether a Python file contains an Airflow DAG definition.
 
     :param file_path: Path to the file to be checked.
-    :param safe_mode: Is safe mode active?. If no, this function always returns True.
     :param zip_file: if passed, checks the archive. Otherwise, check local filesystem.
     :return: True, if file might contain DAGs.
     """
-    if not safe_mode:
-        return True
     if zip_file:
         with zip_file.open(file_path) as current_file:
             content = current_file.read()
     else:
         if zipfile.is_zipfile(file_path):
             return True
-        with open(file_path, 'rb') as dag_file:
+        with open(file_path, "rb") as dag_file:
             content = dag_file.read()
     content = content.lower()
-    return all(s in content for s in (b'dag', b'airflow'))
+    return all(s in content for s in (b"dag", b"airflow"))
+
+
+def _find_imported_modules(module: ast.Module) -> Generator[str, None, None]:
+    for st in module.body:
+        if isinstance(st, ast.Import):
+            for n in st.names:
+                yield n.name
+        elif isinstance(st, ast.ImportFrom) and st.module is not None:
+            yield st.module
+
+
+def iter_airflow_imports(file_path: str) -> Generator[str, None, None]:
+    """Find Airflow modules imported in the given file."""
+    try:
+        parsed = ast.parse(Path(file_path).read_bytes())
+    except Exception:
+        return
+    for m in _find_imported_modules(parsed):
+        if m.startswith("airflow."):
+            yield m
