@@ -24,10 +24,11 @@ from __future__ import annotations
 import copy
 import functools
 import operator
-import re
 import weakref
 from typing import TYPE_CHECKING, Any, Generator, Iterator, Sequence
 
+import re2
+
 from airflow.compat.functools import cache
 from airflow.exceptions import (
     AirflowDagCycleException,
@@ -140,6 +141,7 @@ class TaskGroup(DAGNode):
 
         if parent_group:
             parent_group.add(self)
+            self._update_default_args(parent_group)
 
         self.used_group_ids.add(self.group_id)
         if self.group_id:
@@ -165,17 +167,21 @@ class TaskGroup(DAGNode):
         if self._group_id in self.used_group_ids:
             if not add_suffix_on_collision:
                 raise DuplicateTaskIdFound(f"group_id '{self._group_id}' has already been added to the DAG")
-            base = re.split(r"__\d+$", self._group_id)[0]
+            base = re2.split(r"__\d+$", self._group_id)[0]
             suffixes = sorted(
-                int(re.split(r"^.+__", used_group_id)[1])
+                int(re2.split(r"^.+__", used_group_id)[1])
                 for used_group_id in self.used_group_ids
-                if used_group_id is not None and re.match(rf"^{base}__\d+$", used_group_id)
+                if used_group_id is not None and re2.match(rf"^{base}__\d+$", used_group_id)
             )
             if not suffixes:
                 self._group_id += "__1"
             else:
                 self._group_id = f"{base}__{suffixes[-1] + 1}"
 
+    def _update_default_args(self, parent_group: TaskGroup):
+        if parent_group.default_args:
+            self.default_args = {**parent_group.default_args, **self.default_args}
+
     @classmethod
     def create_root(cls, dag: DAG) -> TaskGroup:
         """Create a root TaskGroup with no group_id or parent."""
