@@ -17,7 +17,10 @@
  * under the License.
  */
 
-import type { DagRun, RunOrdering, Task } from 'src/types';
+import Color from "color";
+import type { DagRun, RunOrdering, Task, TaskInstance } from "src/types";
+
+import useOffsetTop from "./useOffsetTop";
 
 // Delay in ms for various hover actions
 const hoverDelay = 200;
@@ -25,34 +28,75 @@ const hoverDelay = 200;
 function getMetaValue(name: string) {
   const elem = document.querySelector(`meta[name="${name}"]`);
   if (!elem) {
-    return '';
+    return "";
   }
-  return elem.getAttribute('content') || '';
+  return elem.getAttribute("content") || "";
 }
 
-const finalStatesMap = () => new Map([
-  ['success', 0],
-  ['failed', 0],
-  ['upstream_failed', 0],
-  ['up_for_retry', 0],
-  ['up_for_reschedule', 0],
-  ['running', 0],
-  ['deferred', 0],
-  ['queued', 0],
-  ['scheduled', 0],
-  ['skipped', 0],
-  ['no_status', 0],
-]);
+const finalStatesMap = () =>
+  new Map([
+    ["success", 0],
+    ["failed", 0],
+    ["upstream_failed", 0],
+    ["up_for_retry", 0],
+    ["up_for_reschedule", 0],
+    ["running", 0],
+    ["deferred", 0],
+    ["queued", 0],
+    ["scheduled", 0],
+    ["skipped", 0],
+    ["no_status", 0],
+  ]);
 
-const appendSearchParams = (url: string | null, params: URLSearchParams | string) => {
-  if (!url) return '';
-  const separator = url.includes('?') ? '&' : '?';
+interface GroupSummaryProps {
+  group: Task;
+  runId?: string;
+  mappedStates: TaskInstance["mappedStates"];
+}
+
+const getGroupAndMapSummary = ({
+  group,
+  runId,
+  mappedStates,
+}: GroupSummaryProps) => {
+  let totalTasks = 0;
+  const childTaskMap = finalStatesMap();
+  if (!!group.children && !group.isMapped) {
+    group.children?.forEach((child) => {
+      const taskInstance = child.instances.find((ti) => ti.runId === runId);
+      if (taskInstance) {
+        const stateKey =
+          taskInstance.state == null ? "no_status" : taskInstance.state;
+        if (childTaskMap.has(stateKey)) {
+          childTaskMap.set(stateKey, (childTaskMap.get(stateKey) || 0) + 1);
+        }
+      }
+    });
+  } else if (group.isMapped && mappedStates) {
+    Object.entries(mappedStates).forEach(([key, value]) => {
+      totalTasks += value;
+      childTaskMap.set(key || "no_status", value);
+    });
+  }
+
+  return {
+    totalTasks,
+    childTaskMap,
+  };
+};
+
+const appendSearchParams = (
+  url: string | null,
+  params: URLSearchParams | string
+) => {
+  if (!url) return "";
+  const separator = url.includes("?") ? "&" : "?";
   return `${url}${separator}${params}`;
 };
 
 interface GetTaskProps {
   task: Task;
-  taskId: Task['id'];
+  taskId: Task["id"];
 }
 
 const getTask = ({ taskId, task }: GetTaskProps) => {
@@ -85,12 +129,16 @@ const getTaskSummary = ({
   let gc = groupCount;
   const op = operators;
   if (task.children) {
-    if (task.id) { // Don't count the root
+    if (task.id) {
+      // Don't count the root
       gc += 1;
     }
     task.children.forEach((c) => {
       const childSummary = getTaskSummary({
-        task: c, taskCount: tc, groupCount: gc, operators: op,
+        task: c,
+        taskCount: tc,
+        groupCount: gc,
+        operators: op,
       });
       if (childSummary) {
         tc = childSummary.taskCount;
@@ -121,15 +169,23 @@ interface RunLabelProps {
 
 const getDagRunLabel = ({
   dagRun,
-  ordering = ['dataIntervalEnd', 'executionDate'],
+  ordering = ["dataIntervalEnd", "executionDate"],
 }: RunLabelProps) => dagRun[ordering[0]] ?? dagRun[ordering[1]];
 
+const getStatusBackgroundColor = (color: string, hasNote: boolean) =>
+  hasNote
+    ? `linear-gradient(-135deg, ${Color(color).hex()}60 5px, ${color} 0);`
+    : color;
+
 export {
   hoverDelay,
   finalStatesMap,
+  getGroupAndMapSummary,
   getMetaValue,
   appendSearchParams,
   getTask,
   getTaskSummary,
   getDagRunLabel,
+  getStatusBackgroundColor,
+  useOffsetTop,
 };
